// Readable.hpp

#ifndef BRYNHILD_CORE_READABLE_HPP
#define BRYNHILD_CORE_READABLE_HPP

#include <stdexcept>
#include "../io/IOException.hpp"

namespace brynhild {
    namespace core {

        /**
         * A Readable is a source of characters. Characters from
         * a Readable are made available to callers of the read
         * method.
         */
        class Readable {
        public:
            // Virtual destructor
            virtual ~Readable() {}

            /**
             * Attempts to read characters into the specified character array.
             * The array is used as a repository of characters as-is: the only
             * changes made are the results of a put operation.
             *
             * @param cbuf The character array to read characters into.
             * @param len The number of characters to read.
             * @return The number of char values added to the array,
             *         or -1 if this source of characters is at its end.
             * @throws IOException if an I/O error occurs.
             */
            virtual int read(char* cbuf, int len) = 0;
        };

    } // namespace core
} // namespace brynhild

#endif // BRYNHILD_CORE_READABLE_HPP
