// Closeable.hpp

#ifndef BRYNHILD_IO_CLOSEABLE_HPP
#define BRYNHILD_IO_CLOSEABLE_HPP

#include "../core/AutoCloseable.hpp" // Assuming AutoCloseable is defined in a separate header

namespace brynhild {
    namespace io {

        /**
         * A Closeable is a source or destination of data that can be closed.
         * The close method is invoked to release resources that the object is
         * holding (such as open files).
         */
        class Closeable : public core::AutoCloseable {
        public:
            /**
             * Virtual destructor. Ensures that derived classes can clean up resources.
             */
            virtual ~Closeable() {}

            /**
             * Closes this stream and releases any system resources associated
             * with it. If the stream is already closed, then invoking this
             * method has no effect.
             *
             * Subclasses should override this method to perform the actual
             * resource release.
             *
             * @throws IOException if an I/O error occurs. In C++, this can be handled
             * by throwing exceptions derived from std::exception.
             */
            virtual void close() override = 0; // Marking it as pure virtual
        };

    } // namespace io
} // namespace brynhild

#endif // BRYNHILD_IO_CLOSEABLE_HPP
