// IOException.hpp

#ifndef BRYNHILD_IO_IOEXCEPTION_HPP
#define BRYNHILD_IO_IOEXCEPTION_HPP

#include <exception>
#include <string>

namespace brynhild {
    namespace io {

        /**
         * Signals that an I/O exception of some sort has occurred. This
         * class is the general class of exceptions produced by failed or
         * interrupted I/O operations.
         */
        class IOException : public std::exception {
        private:
            std::string message;

        public:
            // Default constructor
            IOException() : message("I/O error occurred") {}

            // Constructor with a specific message
            IOException(const std::string& msg) : message(msg) {}

            // Constructor with a specific message and a cause
            IOException(const std::string& msg, const std::exception& cause)
                : message(msg + ": " + cause.what()) {}

            // Constructor with a cause
            IOException(const std::exception& cause)
                : message("I/O error caused by: " + std::string(cause.what())) {}

            // Override the what() method
            virtual const char* what() const noexcept override {
                return message.c_str();
            }
        };

    } // namespace io
} // namespace brynhild

#endif // BRYNHILD_IO_IOEXCEPTION_HPP
