// InputStreamReader.hpp

#ifndef BRYNHILD_IO_INPUTSTREAMREADER_HPP
#define BRYNHILD_IO_INPUTSTREAMREADER_HPP

#include "Reader.hpp"
#include "InputStream.hpp"
#include "IOException.hpp"

namespace brynhild {
    namespace io {

        /**
         * InputStreamReader is a bridge from byte streams to character streams.
         * It reads bytes from an InputStream and converts them into characters.
         */
        class InputStreamReader : public Reader {
        private:
            InputStream& inStream;  // Reference to the InputStream

        public:
            // Constructor with InputStream
            explicit InputStreamReader(InputStream& in) : inStream(in) {}

            // Override read methods from Reader
            int read(char* cbuf, int off, int len) override {
                if (cbuf == nullptr) {
                    throw IOException("Buffer is null");
                }
                if (len <= 0 || off < 0) {
                    return 0;
                }

                int bytesRead = 0, byteRead;
                for (int i = 0; i < len; ++i) {
                    byteRead = inStream.read();
                    if (byteRead == -1) {
                        break;
                    }
                    cbuf[off + i] = static_cast<char>(byteRead);
                    bytesRead++;
                }

                return bytesRead == 0 && len > 0 ? -1 : bytesRead;
            }

            // Implement the close method
            void close() override {
                inStream.close();
            }
        };

    } // namespace io
} // namespace brynhild

#endif // BRYNHILD_IO_INPUTSTREAMREADER_HPP
