// SocketInputStream.hpp

#ifndef BRYNHILD_IO_SOCKETINPUTSTREAM_HPP
#define BRYNHILD_IO_SOCKETINPUTSTREAM_HPP

#include "InputStream.hpp"
#include "IOException.hpp"
#include <string>
#include <mutex>

namespace brynhild {
namespace io {

class SocketInputStream : public InputStream {
private:
    std::string serverIP;
    int dataPort;
    int controlPort;
    std::mutex mutex;

public:
    // Constructor with server IP/domain, data port, and control port
    SocketInputStream(const std::string& serverIP, int dataPort, int controlPort)
        : serverIP(serverIP), dataPort(dataPort), controlPort(controlPort) {
        // Constructor logic (e.g., establishing socket connection)
    }

    // Override read method (abstract)
    int read() override = 0;

    // Override close method
    void close() override {
        std::lock_guard<std::mutex> lock(mutex); // Lock for thread safety
        // Close logic for the socket stream
    }

    // Other InputStream methods remain abstract and should be implemented by derived classes
};

} // namespace io
} // namespace brynhild

#endif // BRYNHILD_IO_SOCKETINPUTSTREAM_HPP
