// Writer.hpp

#ifndef BRYNHILD_IO_WRITER_HPP
#define BRYNHILD_IO_WRITER_HPP

#include "../core/Appendable.hpp"
#include "Flushable.hpp"
#include "Closeable.hpp"
#include <string>
#include <mutex>
#include <vector>
#include <stdexcept>

namespace brynhild {
namespace io {

class Writer : public core::Appendable, public Flushable, public Closeable {
protected:
    std::mutex lock;

public:
    Writer() = default;
    virtual ~Writer() {}

    // Pure virtual functions
    virtual void write(const char* cbuf, size_t off, size_t len) = 0;
    virtual void flush() = 0;
    virtual void close() = 0;

    // Write a single character
    virtual void write(char c) {
        std::lock_guard<std::mutex> guard(lock);
        write(&c, 0, 1);
    }

    // Write a string
    virtual void write(const std::string& str) {
        write(str.c_str(), 0, str.size());
    }

    // Appendable interface methods
    virtual Writer& append(const std::string& csq, int start = 0, int end = -1) override {
        if (end == -1) end = csq.size();
        if (start < 0 || start > end || end > static_cast<int>(csq.size())) {
            throw std::out_of_range("Invalid start or end position");
        }
        write(csq.substr(start, end - start));
        return *this;
    }

    virtual Writer& append(char c) override {
        write(c);
        return *this;
    }
};

} // namespace io
} // namespace brynhild

#endif // BRYNHILD_IO_WRITER_HPP
